@@ -354,6 +354,15 @@ gss_init_sec_context (OM_uint32 * minor_status,
   _gss_mech_api_t mech;
   int freecontext = 0;
 
+  if (output_token)
+    {
+      output_token->length = 0;
+      output_token->value = NULL;
+    }
+
+  if (ret_flags)
+    *ret_flags = 0;
+
   if (!context_handle)
     {
       if (minor_status)
@@ -781,9 +790,6 @@ gss_delete_sec_context (OM_uint32 * minor_status,
 
   ret = mech->delete_sec_context (NULL, context_handle, output_token);
 
-  if ((*context_handle)->peerptr != GSS_C_NO_NAME)
-    gss_release_name (NULL, &(*context_handle)->peerptr);
-
   free (*context_handle);
   *context_handle = GSS_C_NO_CONTEXT;
 
