@@ -26,20 +26,17 @@
 #include <stddef.h>
 
 /* See version.c. */
-extern const char *
-gss_check_version (const char *req_version);
+extern const char *gss_check_version (const char *req_version);
 
 /* See ext.c. */
-extern int
-gss_oid_equal (gss_OID first_oid, gss_OID second_oid);
+extern int gss_oid_equal (gss_OID first_oid, gss_OID second_oid);
 extern OM_uint32
 gss_copy_oid (OM_uint32 * minor_status,
 	      const gss_OID src_oid, gss_OID dest_oid);
 extern OM_uint32
 gss_duplicate_oid (OM_uint32 * minor_status,
 		   const gss_OID src_oid, gss_OID * dest_oid);
-extern int
-gss_userok (const gss_name_t name, const char *username);
+extern int gss_userok (const gss_name_t name, const char *username);
 
 /* See asn1.c. */
 extern int
@@ -52,6 +49,11 @@ gss_encapsulate_token_prefix (gss_buffer_t input_message,
 extern int
 gss_decapsulate_token (gss_buffer_t input_message,
 		       gss_OID token_oid, gss_buffer_t output_message);
+extern int
+gss_decapsulate_token_check (gss_buffer_t input_message,
+			     char *prefix, size_t prefixlen,
+			     gss_OID token_oid,
+			     gss_buffer_t output_message);
 
 /* Static versions of the public OIDs for use, e.g., in static
    variable initalization.  See oid.c. */
