@@ -44,8 +44,7 @@ OM_uint32
 gss_krb5_verify_mic (OM_uint32 * minor_status,
 		     const gss_ctx_id_t context_handle,
 		     const gss_buffer_t message_buffer,
-		     const gss_buffer_t token_buffer,
-		     gss_qop_t * qop_state)
+		     const gss_buffer_t token_buffer, gss_qop_t * qop_state)
 {
   return GSS_S_UNAVAILABLE;
 }
